<?php

/**
 * @package redaxo\arthelper
 *
 * @internal
 */
abstract class rex_arthelper {
    private static function _getArguments(array $args) {
        $param = array();

        if(!empty($args) && is_array($args)) {
            foreach($args as $arg) {
                if(array_keys($arg) !== range(0, count($arg) - 1)) {
                    $param = array_merge($param, $arg);
                }
                else if(!isset($param['article_id']) && !isset($param['clang'])) {
                    $arg = (int) $arg;
                    if(!is_nan($arg) && $arg > 0) {
                        if(!isset($param['article_id'])) {
                            $param['article_id'] = $arg;
                        }
                        else if(!isset($param['clang'])) {
                            $param['clang'] = $arg;
                        }
                    }
                }
            }
        }

        if(empty($param['article_id'])) {
            $param['article_id'] = rex_article::getCurrentId();
        }

        if(empty($param['clang'])) {
            $param['clang'] = rex_clang::getCurrentId();
        }

        return $param;
    }
    
    public static function getHtmlTag() {
        $args = self::_getArguments(func_get_args());

        if(!isset($args['class'])) {
            $args['class'] = array();
        }
        else {
            if(!is_array($args['class'])) {
                $args['class'] = (array) $args['class'];
            }
            else {
                $args['class'] = array_values($args['class']);
            }
        }

        if(is_object($art = rex_article::get($args['article_id'], $args['clang']))) {

            $args['class'][] = 'rex-article-' . $art->getId();
            
            if($pid = $art->getParentId()) {
                $args['class'][] = 'child-of-' . strval($pid);
            }

            if($art->isStartArticle()) {
                $args['class'][] = 'is-category';
            }

            if(rex_article::getSiteStartArticleId() == $art->getId()) {
                $args['class'][] = 'is-site-start-article';
            }

            if($art->getValue('template_id')) {
                $args['class'][] = 'rex-template-' . $art->getValue('template_id');
            }

            // lang
            if(!isset($args['lang']) OR !is_string($args['lang'])) {
                $args['lang'] = rex_clang::get($args['clang'])->getCode();
            }
        }

        $args['class'] = trim(join(' ', array_unique(array_filter($args['class']))));


        if(!isset($args['lang']) OR !is_string($args['lang'])) {
            $args['lang'] = rex_clang::getCurrent()->getCode();
        }

        unset($args['article_id'], $args['clang']);

        if(!empty($args)) {
            foreach($args as $k=>&$v) {
                $v = $k . '="' . $v . '"';
            }
        }
    
        $out = '<html' . (!empty($args) ? ' '.join(' ',$args) : '') . '>';

        return $out;
    }

    public static function getArticleCss($article, &$csslist = null) {
        if(!is_object($article)) {
            return null;
        }

        if(!is_array($csslist)) {
            $csslist = array();
        }

        $artkey = 'rex-article-' . $article->getId();
        $clang = $article->getClang();

        if(isset($csslist[$artkey])) {
            return $csslist[$artkey];
        }

        if($clang != 1 && (bool) $article->getValue('arthelper_use_root')) {
            $article = rex_article::get($article->getId(), 1);
        }

        $article_css = str_replace(',', ' ',$article->getValue('arthelper_css'));
        
        if(!(bool) $article->getValue('arthelper_ignore_pcss') && is_object($pnt = rex_article::get($article->getParentId(), $clang))) {
            $article_css.= ' ' . join(' ', self::getArticleCss($pnt, $csslist));
        }

        // replace special chars by alphanumeric representatives
        $article_css = iconv("UTF-8", "ASCII//TRANSLIT", trim($article_css));
        $article_css = preg_replace('/["\']/i', '', $article_css);
        $article_css = preg_replace('/[^a-z0-9\-\_ ]/i', '_', $article_css);
        $article_css = @array_unique(array_filter(explode(' ', $article_css)));

        $csslist[$artkey] = $article_css;

        unset($article_css);

        return $csslist[$artkey];
    }

    public static function replaceArticleCss (rex_extension_point $ep) {
        $subject = $ep->getSubject();

        // check the content for class="" attributes that contain rex-article-XXX ...
        preg_match_all('/class="([^"]+)? ?(rex-article-[0-9]+)([ |$]+)?([^"]+)?"/', $subject, $matches, PREG_SET_ORDER);
        if(!empty($matches[0])) {
            $articles = array();
            $replacements = array();

            foreach($matches as $i=>$article_css) {
                $src = array_shift($article_css);
                $article_css = join(' ',$article_css);
                $article_css = array_unique(array_filter(explode(' ',$article_css)));

                preg_match_all('/rex-article-(\d+)/', join(' ',$article_css), $article_ids);
                if(!empty($article_ids[0])) {
                    $articles = array_merge($articles, $article_ids[0]);

                    $replacements[] = array(
                        'src' => $src,
                        'css' => $article_css
                    );
                }
                unset($article_ids);
            }
            unset($src,$article_css,$i);

            if(!empty($articles)) {
                $csslist = array();
                $articles = array_unique($articles);

                foreach($articles as $article) {
                    rex_arthelper::getArticleCss(rex_article::get((int) substr($article,strrpos($article,'-')+1)), $csslist);
                }
                unset($article);

                // remove empty CSS fields
                $csslist = array_filter($csslist);
                if(!empty($csslist)) {
                    foreach($replacements as $replacement) {
                        $this_css = $replacement['css'];
                        foreach($replacement['css'] as $single_css) {
                            if(!empty($csslist[$single_css])) {
                                $this_css = array_merge($this_css, $csslist[$single_css]);
                            }
                        }
                        $this_css = array_unique($this_css);
                        sort($this_css);
                        $replacement['dst'] = 'class="' . join(' ',$this_css) . '"';
                        unset($this_css);

                        $subject = str_replace($replacement['src'], $replacement['dst'], $subject);
                    }
                    unset($replacement);
                }
                unset($csslist);
            }
            unset($articles,$replacements);
        }
        unset($matches);

        return $subject;    
    }

    public static function addBackendSidebar (rex_extension_point $ep) {
        $params = $ep->getParams();
        $subject = $ep->getSubject();

        $panel = include(rex_path::addon('arthelper','pages/content.arthelper.sidebar.php'));

        $fragment = new rex_fragment();
        $fragment->setVar('title', '<i class="rex-icon rex-icon-info"></i> ' . rex_i18n::msg('arthelper_extras'), false);
        $fragment->setVar('body', $panel, false);
        $fragment->setVar('article_id', $params["article_id"], false);

        $fragment->setVar('collapse', true);
        $fragment->setVar('collapsed', true);
        $content = $fragment->parse('core/page/section.php');

        return $subject.$content;

    }
}